#!@@SH@@
#
# pro-archman
# src/pro-archman.sh
# Main program file
#
# Copyright (C) 2013  Patrick "P. J." McDermott
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

set -u

# Constant global variables
PACKAGE_NAME='@@PACKAGE_NAME@@'
PACKAGE_VERSION='@@PACKAGE_VERSION@@'
PKGLIBDIR='@@PKGLIBDIR@@'
PKGLIBCMDDIR='@@PKGLIBCMDDIR@@'
PKGLIBCMD='@@PKGLIBCMD@@'
LF='
'
OPTSTRING='hVb:'

# Global variables
loading_cmd=
base_dir=
conf_incoming_channel=
conf_incoming_dir=
conf_pool_gc_delay=
lock=

# use() must be defined inline so it can be used to load other modules.
use()
{
	local module="${1}"
	local default_dir=
	local lib_subdir=
	local dir=

	if [ "x${module%/*}" = "x${module}" ]; then
		default_dir="${PKGLIBDIR}"
		lib_subdir=''
	else
		case "${module%/*}" in
			'cmd')
				loading_cmd="$(printf '%s' "${module##*/}" | \
					tr '[A-Z]' '[a-z]' | \
					tr -C '[a-z0-9_]' '_')"
				default_dir="${PKGLIBCMDDIR}"
				lib_subdir='cmd'
				;;
		esac
	fi

	if [ "${ARCHMAN_LIBDIR+set}" = 'set' ]; then
		dir="${ARCHMAN_LIBDIR:-.}/${lib_subdir}"
	else
		dir="${default_dir}"
	fi

	if [ -f "${dir}/${module##*/}.sm" ]; then
		. "${dir}/${module##*/}.sm"
	else
		printf '%s: Error: Failed to load module "%s": %s\n' \
			"${0##*/}" "${module}" 'no such file or directory' >&2
		exit 2
	fi
}

use locale
use cmd
use index
use garbage

main()
{
	local cmd=
	local status=

	load_locale
	load_cmds

	get_options "${@}"
	shift $(($OPTIND - 1))

	base_dir="${opt_b:-.}"

	if ${opt_h:-false}; then
		cmd='help'
	elif ${opt_V:-false}; then
		cmd='version'
	elif [ ${#} -lt 1 ]; then
		cmd_help_main >&2
		exit 1
	else
		cmd="${1}"
		shift
	fi

	run_cmd "${cmd}" "${@}"
	status=${?}

	return ${status}
}

get_options()
{
	local opt=

	unset OPTARG
	while getopts "${OPTSTRING}" opt; do
		eval "opt_${opt}=\"\${OPTARG:-true}\""
		unset OPTARG
	done

	return 0
}

init()
{
	lock
	get_conf
}

lock()
{
	mkdir -p "${base_dir}/.db"
	lock="${base_dir}/.db/lock"
	if ! (set -C; printf '%d\n' "${$}" >"${lock}") 2>/dev/null; then
		error 2 "$(get_msg 'lock_fail')"
	fi
}

get_conf()
{
	local old_dir=

	# Set defaults.
	conf_incoming_channel='dev'
	conf_incoming_dir='../incoming'
	conf_pool_gc_delay=86400

	if [ -f "${base_dir}/conf" ]; then
		. "${base_dir}/conf"
	fi

	old_dir="$(pwd)"
	cd "${base_dir}"
	if [ -d "${conf_incoming_dir}" ]; then
		conf_incoming_dir="$(cd "${conf_incoming_dir}" && pwd)"
	fi
	cd "${old_dir}"

	return 0
}

fini()
{
	update_feeds
	collect_garbage

	rm -f "${lock}"
}

main "${@}"
